# -*- coding: utf-8 -*-
# @package Download Image Sentinel
"""
Install:

        $ sudo pip3 install sentinelsat
	$ sudo apt-get install libgdal-dev

Example:

        $ python3 -m doctest -v APISentinel.py
"""

import os
from sentinelsat.sentinel import SentinelAPI
from datetime import date
from collections import OrderedDict
import time
#from osgeo import ogr
'C4+rC@W>7C5s$b'

class APISentinel(object):
	""" Class for Sentinel satellites configuration
	Test Case
	>>> sentinel = APISentinel('mario-chirinos', 'r4nc0r4u')
	>>> products = sentinel.getProducts("POLYGON((-89.99450683593753 21.279137394108716,-89.13757324218751 21.2996106049456,-89.30236816406251 20.68418377935238,-90.0494384765625 20.715015145512098,-89.99450683593753 21.279137394108716))", ('20151219', date(2015,12,29)), {"platformname":"Sentinel-2"})
	>>> print(len(products))
	6

	"""

	def __init__(self,usernam,passw):
		"""The constructor Initialize Sentinel Data.

		Args:
		    self: The object pointer.
		    usernam (str): Username, access credentials to https://scihub.copernicus.eu/dhus/#/home.
		    passw (str): Password, access credentials to https://scihub.copernicus.eu/dhus/#/home.

		Returns:
		    pointer: The object pointer.
		"""
		self.username = usernam
		self.password = passw
#		self.URL_Sentinel = 'https://scihub.copernicus.eu/dhus'
#		self.URL_Sentinel = 'https://scihub.copernicus.eu/apihub'
		self.URL_Sentinel = 'https://apihub.copernicus.eu/apihub/'
	# self.plot_width = 7
	# self.plot_height = 7
	# self.intersection_th = 30
		self.api = SentinelAPI(self.username, self.password, self.URL_Sentinel)

	def getProducts(self, area, date, searchParameters, areaRelation="Intersects"):
		"""Gets Sentinel products list that match the search parameters.

		Args:
			self (pointer): The object pointer.
			area (str): Area of interest using a Polygon in WKT format.
			date (tuple): A tuple contining the initial and end dates. 
			searchParameters (dict): Additional keywords can be used to specify other query parameters, 
				Other Parameters: Additional keywords can be used to specify other query parameters,
				e.g. {"platformname":"Sentinel-2", "cloudcoverpercentage":"[0 TO 40]"} 
				See https://scihub.copernicus.eu/twiki/do/view/SciHubUserGuide/3FullTextSearch for a full list. Ran-
				ge values can be passed as two-element tuples, e.g. cloudcoverpercentage=(0, 30).
				The time interval formats accepted by the `date` parameter can also be used with any other parameters that expect time intervals
				(that is: 'beginposition', 'endposition', 'date', 'creationdate', and 'ingestiondate').
		Example:
			getProducts("POLYGON((-89.99 21.27,-89.13 21.29,-89.30 20.68,-90.04 20.71,-89.99 21.27))", ('20150101', '20181024'), {"platformname":"Sentinel-2", "cloudcoverpercentage":"[0 TO 40]"})

		Returns:
			OrderedDict: Sentinel Product list found.

		Notes:
			area_relation : {'Intersects', 'Contains', 'IsWithin'}, optional
			What relation to use for testing the AOI. Case insensitive.
			- Intersects: true if the AOI and the footprint intersect (default)
			- Contains: true if the AOI is inside the footprint
			- IsWithin: true if the footprint is inside the AOI

		"""



		products_list = self.api.query(area, date, area_relation=areaRelation, **searchParameters)
		return products_list

	def downloadProducts(self,products,dir):
		"""Download product(s) list Sentinel

			Args:
				self (pointer): The object pointer.
				products (OrderedDict): Product(s) to download.
				dir (str): destination directory.
		"""

		os.chdir(dir)
		#        self.api.download_all(products)
		longTermArchiveList=[]
		for p in products:
			print(products[p]['filename'])
			try:
				info = self.api.download(p)
				if not info["Online"]:
					print("Adding producto to long term list")
					longTermArchiveList.append(p)
					time.sleep(1)
			except:	

				print ("ERROR")

				
		return longTermArchiveList

#		def filterProducts(self, productList):
#		#, user_footprint, threshold):

#			products = productList.copy()
#			for p in productList:
#				if productList[p]['filename'].find("OPER_PRD") != -1:
#					del products[p]


#			return products


