#!/usr/bin/python3
# -*- coding: utf-8 -*-

import sys, os
import json

if __name__ == "__main__": # Local Run
    import APISentinel
    import polygonToBox
else: # Module Run, When going production - delete if/else
    from . import APISentinel
    from . import polygonToBox

from osgeo import ogr
from datetime import date 
from collections import OrderedDict

productType = {"L1C":"S2MSI1C", "L2A":"S2MSI2A"}
#===============================================================================
def findSentinelProducts(cfg, uinput = False, limitto = 0):#main(argv):

	sentinel = APISentinel.APISentinel(cfg['username'],  cfg['password'])
	geometry = ogr.CreateGeometryFromWkt(cfg['wkt'])

	wktList = []
	if geometry.GetGeometryType() == ogr.wkbPolygon:
		print("POLYGON")
		wkt = polygonToBox.getWKTPolygonBoundingBox(cfg['wkt'], True)
		wktList.append(wkt)
		print(wkt)
	elif geometry.GetGeometryType() == ogr.wkbMultiPolygon:
		print("MULTIPOLYGON")
		for i in range(0, geometry.GetGeometryCount()):
			g = geometry.GetGeometryRef(i).ExportToWkt()
			wkt = polygonToBox.getWKTPolygonBoundingBox(g, True)
			print(wkt)	
			wktList.append(wkt)

	productList = OrderedDict()
	for wkt in wktList:
		tmpList	 =sentinel.getProducts(wkt, (cfg['startDate'], cfg['endDate']), {"producttype":productType[cfg["productLevel"]], "platformname":cfg['platform'], "cloudcoverpercentage":"[0 TO "+str(cfg['clouds'])+"]"}, areaRelation="Contains")		

		if len(productList)<=0:
			print ("No products found with 'Contains' trying with 'Intersects'")
			tmpList = sentinel.getProducts(wkt, (cfg['startDate'], cfg['endDate']), {"producttype":productType[cfg["productLevel"]], "platformname":cfg['platform'], "cloudcoverpercentage":"[0 TO "+str(cfg['clouds'])+"]"})
	
		productList.update(tmpList)

	if limitto>0:
		tilesdic={}
		for k, v in productList.items():
#			print("productList v", v)
			tile = v["title"][38:44]
			date = v["title"][11:19]
			if tile not in tilesdic:
				tilesdic[tile]={}
			tilesdic[tile][k]=v

		for k,v in tilesdic.items():
			tilesdic[k] = {i[0]:i[1] for i in sorted(v.items(), key=lambda x: x[1]["title"][11:19], reverse=True)}

		productList = {list(tilesdic[k].keys())[0]: list(tilesdic[k].items())[0][1] for k in tilesdic.keys()}

	fileNames = [productList[k]['filename'].replace("SAFE", "zip") for k in productList.keys() ]

	downloadDir =  cfg["productsDir"]+cfg["productLevel"]+"/"
	dirList = os.listdir(downloadDir)

	matchingProducts = set(fileNames).intersection(set(dirList))
	print ( str(len(matchingProducts))+" of " + str(len(fileNames)) +" "+ cfg["productLevel"] + " products found.")

	
	text =""
	if uinput==True:
		while text != "yes" and text != "no":
				text = input("Do you want to download this products to "+downloadDir+ " ? (yes, no)")
	if uinput==False or text=="yes":
		longTermArchiveList = sentinel.downloadProducts(productList,downloadDir)
		if len(longTermArchiveList)>0:
			print( str(len(longTermArchiveList)) + " products are in the long term archive" )
			with open(cfg["projectDir"]+'longTermArchiv.json', 'w') as outfile:
				json.dump(longTermArchiveList, outfile)
	
	L2ADir = cfg["productsDir"]+"L2A/"
	if cfg["productLevel"]=="L1C":
		fileNamesL2A = [productList[k]['filename'].replace("SAFE", "zip").replace("L1C", "L2A") for k in productList.keys() ]
		L1CDir = cfg["productsDir"]+"L1C/"
		linksDir=cfg["projectDir"]+"L1C/"
		if not os.path.exists(linksDir):
			os.mkdir(linksDir)
		L1CDirList = os.listdir(L1CDir)

		#LinkProducts
		fileNames = [productList[k]['filename'].replace("SAFE", "zip") for k in productList.keys() ]
		matchingProducts = set(fileNames).intersection(set(L1CDirList))
		for f in matchingProducts:
			print (f)
			print("ln -s " + L1CDir+f + " " + linksDir+f)

			os.system("ln -s " + L1CDir+f + " " + linksDir+f)
		print (str(len(matchingProducts)) + " Linked to " + linksDir)

		print ( str(len(matchingProducts))+" of " + str(len(fileNames)) +" L1C products found in L2A.")
		text =""
		if uinput==True:
			while text != "yes" and text != "no":
				text = input("Do you want to convert this products and save them to "+downloadDir+ " ? (yes, no)")
		if uinput==False or text=="yes":
			#print("L1CProductListToL2A.sh "+rawDir+" "+inDir+" 1")
			os.system("L1CProductListToL2A.sh "+linksDir+" "+L2ADir+" 1") 

	text =""
	linksDir=cfg["projectDir"]+"L2A/"
	if uinput==True:
		while text != "yes" and text != "no":
				text = input("Do you want to link this products to "+linksDir+ " ? (yes, no)")

	if uinput==False or  text=="yes":
		fileNames = [productList[k]['filename'].replace("SAFE", "zip").replace("L1C", "L2A") for k in productList.keys() ]
		dirList = os.listdir(L2ADir)

		matchingProducts = set(fileNames).intersection(set(dirList))

		for f in matchingProducts:
			print (f)
			os.system("ln -s " + L2ADir+f + " " + linksDir+f)
			print (str(len(matchingProducts)) + " Linked to " + linksDir)

#===============================================================================
def main(argv):
	if len(sys.argv) != 2:
		print("Usage: " + argv[0] + " <JSON File>")
	else:
		with open(argv[1]) as jsonFile:
			cfg = json.load(jsonFile)
			findSentinelProducts(cfg, uinput=True)
if __name__ == "__main__":
	main(sys.argv)


#cat results2.txt | xargs -I{} ln -s {} ~/newlinks

